use ExtUtils::MakeMaker qw(prompt WriteMakefile);

my $QUIET = 0;

# Default install path
my $bin_dir = '/usr/local/sysusage/bin';
my $conf_dir = '/usr/local/sysusage/etc';
my $pid_dir = '/usr/local/sysusage/etc';
my $base_dir = '/usr/local/sysusage/rrdfiles';
my $html_dir = '/var/www/htdocs/sysusage';


# Try to find all binary used by SysUsage
my $hostname = `which hostname`;
chomp($hostname);
if (!$hostname) {
	die "NOTICE: This tool requires system command: hostname, can't find it in your path\n";
}
my $sar = `which sar`;
chomp($sar);
if (!$sar) {
	die "NOTICE: This tool requires the sysstat package, can't find sar in your path\n";
}
my $uptime = `which uptime`;
chomp($uptime);
if (!$uptime) {
	die "NOTICE: This tool requires system command: uptime, can't find it in your path\n";
}

if (grep(/QUIET/i, @ARGV)) {
	print "Don't ask anything. Just create default configuration.\n";
	# Setup ok. generating sysusage.cfg config file
	unless(open(OUTCFG, ">sysusage.cfg")) {
		print "\nError: can't write config file sysusage.cfg, $!\n";
		exit 0;
	}

	print OUTCFG qq{
# This part configure the default settings of Sysusage perl scripts
[GENERAL]
DEBUG     = 0
DATA_DIR  = $base_dir
PID_FILE  = $pid_dir
DEST_DIR  = $html_dir
SAR_BIN   = $sar
UPTIME    = $uptime
HOSTNAME  = $hostname
INTERVAL  = 60
SKIP      = 
HDDTEMP_BIN =

# This part enable/disable and configure smtp alarm report and/or Nagios
# message report (path to nagios binary submit_check_result) when threshold
# min/max exceed are detected. Upper and lower level values are use to set
# the alarm level. With Nagios 0=OK, 1=WARNING, 2=Critical, 3=Unknown.
# Default is to send warning on high value and critical on low threshold value.
# Typically a high load average can be seen as a warning and the shutdown of
# any monitored daemon as critical.
[ALARM]
WARN_MODE   = 0
ALARM_PROG  = $bin_dir/sysusagewarn
SMTP        = localhost
FROM        = root\@localhost
TO          = root\@localhost
NAGIOS      =
UPPER_LEVEL = 1
LOWER_LEVEL = 2

# This part allow system monitoring. There's three format. The first one is
# the most used:
#
#	type:ThresholdMax:ThresholdMin
#
# where type is the type of monitoring.
# The second format is only used for process and queue monitoring:
#
#	type:what:ThresholdMax:ThresholdMin
#
# where type can be 'proc' or 'queue' and what is the related thing to monitor.
# If type is 'proc' then what is a process name, if type is 'queue' then what
# is a directory path to monitor.
#
# The last one is used for disk space monitoring:
#
#       disk:ThresholdMax:exclusion
#
# where exclusion is a semicolon (;) separated list of mount point to exclude
# from monitoring.
#
# A ThresholdMax value set to 0 or omitted mean disable warning
# Omitting ThresholdMin disable warning
#
[MONITOR]
load:
cpu:
wait:
cswch:
intr:
mem:
swap:
sock:
io:
file:
page:
pcrea:
pswap:
net:
err:
disk:
};
	close(OUTCFG);
} else {
	$bin_dir = prompt('Where to install sysusage programs?', $bin_dir);
	$conf_dir = prompt('Where to install configuration file?', $conf_dir);
	$pid_dir = prompt('Where to install pid files?', $pid_dir);
	$base_dir = prompt('Where to install RRD database files?', $base_dir);
	$html_dir = prompt('Where do you want HTML output?', $html_dir);

	# Ask things to monitor
	my $disk = lc prompt('Do you want to monitor disk space?', 'y') eq 'y';
	my $net = lc prompt('Do you want to monitor network traffic?', 'y') eq 'y';
	my $err = lc prompt('Do you want to monitor network errors?', 'y') eq 'y';
	my $load = prompt('Do you want to monitor load average?', 'y') eq 'y';
	my $cpu = prompt('Do you want to monitor cpu utilization?', 'y') eq 'y';
	my $intr = prompt('Do you want to monitor interrupts utilization?', 'y') eq 'y';
	my $cswch = prompt('Do you want to monitor context switches utilization?', 'y') eq 'y';
	my $swap = prompt('Do you want to monitor swap utilization?', 'y') eq 'y';
	my $sock = prompt('Do you want to monitor number of open socket?', 'y') eq 'y';
	my $io = prompt('Do you want to monitor I/O request/page?', 'y') eq 'y';
	my $pcrea = prompt('Do you want to monitor number of process per second creation?', 'y') eq 'y';
	my $file = prompt('Do you want to monitor percentage of open file regarding file-max?', 'y') eq 'y';
	# What process to monitor
	my @proc = ();
	if (lc prompt('Do you want to monitor some process?', 'y') eq 'y') {
		print "What process do you want to monitor (ex:httpd,slapd,sendmail) ?: ";
		my $process = <>;
		chomp($process);
		@proc = split(/\s*,\s*/, $process);
		map { s/\s+// } @proc;
	}

	# What queue directory to monitor
	my @queue = ();
	if (lc prompt('Do you want to monitor some queue directory?', 'y') eq 'y') {
		print "What queue do you want to monitor (ex:/var/spool/mqueue,/var/spool/lpd/) ?: ";
		my $q = <>;
		chomp($q);
		@queue = split(/\s*,\s*/, $q);
		map { s/\s+// } @queue;
	}
	foreach my $n (@queue) {
		if (!-d $n) {
			print "\nError: monitoring queue $n is not a directory.\n"; 
			print "\nAborting install...\n";
			exit 0;
		}
	}

	# What device to monitor
	my @dev = ();
	if (lc prompt('Do you want to CPU usage of some device?', 'y') eq 'y') {
		print "What device do you want to monitor (ex:sda,sdb) ?: ";
		my $devices = <>;
		chomp($devices);
		@dev = split(/\s*,\s*/, $devices);
		map { s/\s+// } @dev;
	}

	# Get memory information
	my $mem = lc prompt('Do you want to monitor memory?', 'y') eq 'y';
	my $work = lc prompt('Do you want to monitor memory needed for workload?', 'y') eq 'y';

	my $hddtemp = '';
	my $hddtemp_path = `which hddtemp`;
	chomp($hddtemp_path);
	if ($hddtemp_path) {
		$hddtemp = lc prompt('Do you want to monitor temperature of your hard drive?', 'y') eq 'y';
		if ($hddtemp) {
			$hddtemp = prompt('Enter devices to monitor separated by colon (ex: /dev/sda,/dev/sdb,...)', '');
		}
	}

	# Should we use threshold warning ?
	my $warn = lc prompt('Do you want to enable threshold warning?', 'y') eq 'y';

	my $thres_load = 15;
	my $thres_cpu = 95;
	my $thres_mem = 99;
	my $thres_work = 90;
	my $thres_swap = 10;
	my $thres_disk = 98;
	my $thres_net = 100000000;
	my $max_proc = 100;
	my $min_proc = 0;
	my $thres_sock = 5000;
	my $thres_io = 5000;
	my $thres_page = 10000;
	my $thres_pswap = 10000;
	my $thres_pcrea = 30;
	my $thres_file = 95;
	my $thres_err = 1;
	my $thres_queue = 100;
	my $thres_cswch = 5000;
	my $thres_intr = 5000;
	my @thres_max_proc = ();
	my @thres_min_proc = ();
	my @thres_max_queue = ();
	my @thres_max_dev = ();
	my $warnme = 0;
	my $dosmtp = 0;
	my $smtp = 'localhost';
	my $from = 'root@localhost';
	my $to = 'root@localhost';
	my $nagios = 0;
	my $nagios_submit_check_result = '';
	my $thres_hddtemp = '30';

	if ($warn) {
		$nagios = lc prompt('Do you want to enable Nagios report?', 'y') eq 'y';
		if ($nagios) {
			$nagios_submit_check_result = prompt('Where is submit_check_result binary?', '/usr/local/nagios/bin/submit_check_result');
			if (!-e "$nagios_submit_check_result") {
				$nagios_submit_check_result = '';
				$nagios = 0;
			}
		}
		$dosmtp = lc prompt('Do you want to enable SMTP report?', 'y') eq 'y';
		if ($dosmtp) {
			$smtp = prompt('What is the address of your SMTP server?', $smtp);
			$from = prompt('What is the sender/from email address?', $from);
			$to = prompt('What is the destination email address?', $to);
		}
		$thres_load = prompt('Load average max threshold (0 mean no threshold)?', $thres_load) if ($load);
		$thres_cpu = prompt('Percentage CPU utilisation max threshold (0 mean no threshold)?', $thres_cpu) if ($cpu);
		$thres_intr = prompt('Number of interrupts per second max threshold (0 mean no threshold)?', $thres_intr) if ($intr);
		$thres_cswch = prompt('Number of context switches per second max threshold (0 mean no threshold)?', $thres_cswch) if ($cswch);
		$thres_mem = prompt('Percentage memory used max threshold (0 mean no threshold)?', $thres_mem) if ($mem);
		$thres_swap = prompt('Percentage swap used max threshold (0 mean no threshold)?', $thres_swap) if ($swap);
		$thres_work = prompt('Percentage memory needed for workload max threshold (0 mean no threshold)?', $thres_work) if ($work);
		$thres_disk = prompt('Percentage disk usage max threshold (0 mean no threshold)?', $thres_disk) if ($disk);
		$thres_net = prompt('Bytes/s network traffic max threshold (0 mean no threshold)?', $thres_net) if ($net);
		$thres_err = prompt('Network packet error max threshold (0 mean no threshold)?', $thres_err) if ($err);
		$thres_sock = prompt('Open socket max threshold (0 mean no threshold)?', $thres_sock) if ($sock);
		$thres_io = prompt('I/O request/s max threshold (0 mean no threshold)?', $thres_io) if ($io);
		$thres_page = prompt ('I/O paging max threshold (0 mean no threshold)?', $thres_page) if ($io);
		$thres_pswap = prompt('I/O pages max swap threshold (0 mean no threshold)?', $thres_pswap) if ($io);
		$thres_pcrea = prompt('Process/s creation max threshold (0 mean no threshold)?', $thres_pcrea) if ($pcrea);
		$thres_file = prompt('Open file percentage max threshold (0 mean no threshold)?', $thres_file) if ($file);
		foreach my $n (@queue) {
			$thres_queue = prompt("Number of files max threshold in queue $n (0 mean no threshold)?", $thres_queue);
			if (!$thres_queue) {
				push(@thres_max_queue, '');
			} else {
				push(@thres_max_queue, $thres_queue);
			}
		}
		foreach my $p (@proc) {
			$max_proc = prompt("Running $p max threshold (0 mean no threshold)?", $max_proc);
			if (!$max_proc) {
				push(@thres_max_proc, '');
			} else {
				push(@thres_max_proc, $max_proc);
			}
			$min_proc = prompt("Running $p min threshold (0 mean warn at process death, -1 to disable)?", $min_proc);
			if ($min_proc < 0) {
				push(@thres_min_proc, '');
			} else {
				push(@thres_min_proc, $min_proc);
			}
		}
		$thres_hddtemp = prompt('Har drive temperature max threshold (0 mean no threshold)?', $thres_hddtemp) if ($hddtemp);
		foreach my $p (@dev) {
			$max_dev = prompt("Percentage of CPU usage for device $p max threshold (0 mean no threshold)?", $max_dev);
			if (!$max_dev) {
				push(@thres_max_dev, '');
			} else {
				push(@thres_max_dev, $max_dev);
			}
		}

	} else {

		foreach (@proc) {
			push(@thres_max_proc, '');
			push(@thres_min_proc, '');
		}
		foreach (@queue) {
			push(@thres_max_queue, '');
		}
		foreach (@dev) {
			push(@thres_max_dev, '');
		}
		$thres_load = '';
		$thres_cpu = '';
		$thres_mem = '';
		$thres_work = '';
		$thres_swap = '';
		$thres_pswap = '';
		$thres_disk = '';
		$thres_sock = '';
		$thres_io = '';
		$thres_page = '';
		$thres_pcrea = '';
		$thres_err = '';
		$thres_file = '';
		$thres_net = '';
		$thres_cswch = '';
		$thres_intr = '';
		$thres_hddtemp = '';
		$warnme = 0;
	}

	my $skip = '';
	if ($warn) {
		$skip = prompt('Want range of time should be removed from alert [ex: 01:50/02:50]?', '');
		while ($skip && ($skip !~ m#\d{2}:\d{2}/\d{2}:\d{2}#)) {
			$skip = prompt('Want range of time should be removed from alert [ex: 01:50/02:50]?', '');
		}
	}
	# Setup ok. generating sysusage.cfg config file
	unless(open(OUTCFG, ">sysusage.cfg")) {
		print "\nError: can't write config file sysusage.cfg, $!\n";
		exit 0;
	}

	print OUTCFG qq{
# This part configure the default settings of Sysusage perl scripts
[GENERAL]
DEBUG       = 0
DATA_DIR    = $base_dir
PID_DIR     = $pid_dir
DEST_DIR    = $html_dir
SAR_BIN     = $sar
UPTIME      = $uptime
HOSTNAME    = $hostname
INTERVAL    = 60
SKIP        = $skip
HDDTEMP_BIN = $hddtemp_path

# This part enable/disable and configure smtp alarm report and/or Nagios
# message report (path to nagios binary submit_check_result) when threshold
# min/max exceed are detected. Upper and lower level values are use to set
# the alarm level. With Nagios 0=OK, 1=WARNING, 2=Critical, 3=Unknown.
# Default is to send warning on high value and critical on low threshold value.
# Typically a high load average can be seen as a warning and the shutdown of
# any monitored daemon as critical.
[ALARM]
WARN_MODE   = $warnme
ALARM_PROG  = $bin_dir/sysusagewarn
SMTP        = $smtp
FROM        = $from
TO          = $to
NAGIOS      = $nagios_submit_check_result
UPPER_LEVEL = 1
LOWER_LEVEL = 2

# This part allow system monitoring. There's three format. The first one is
# the most used:
#
#	type:ThresholdMax:ThresholdMin
#
# where type is the type of monitoring.
# The second format is only used for process and queue monitoring:
#
#	type:what:ThresholdMax:ThresholdMin
#
# where type can be 'proc' or 'queue' and what is the related thing to monitor.
# If type is 'proc' then what is a process name, if type is 'queue' then what
# is a directory path to monitor.
#
# The last one is used for disk space monitoring:
#
#       disk:ThresholdMax:exclusion
#
# where exclusion is a semicolon (;) separated list of mount point to exclude
# from monitoring.
#
# A ThresholdMax value set to 0 or omitted mean disable warning
# Omitting ThresholdMin disable warning
#
[MONITOR]
};
	print OUTCFG "load:$thres_load\n" if ($load);
	print OUTCFG "cpu:$thres_cpu\n" if ($cpu);
	print OUTCFG "wait:0\n" if ($cpu);
	print OUTCFG "cswch:$thres_cswch\n" if ($cswch);
	print OUTCFG "intr:$thres_intr\n" if ($intr);
	print OUTCFG "mem:$thres_mem\n" if ($mem);
	print OUTCFG "swap:$thres_swap\n" if ($swap);
	print OUTCFG "work:$thres_work\n" if ($work);
	print OUTCFG "sock:$thres_sock\n" if ($sock);
	print OUTCFG "io:$thres_io\n" if ($io);
	print OUTCFG "file:$thres_file\n" if ($file);
	print OUTCFG "page:$thres_page\n" if ($file);
	print OUTCFG "pcrea:$thres_pcrea\n" if ($pcrea);
	print OUTCFG "pswap:$thres_pswap\n" if ($io);
	print OUTCFG "net:$thres_net\n" if ($net);
	print OUTCFG "err:$thres_err\n" if ($err);
	print OUTCFG "disk:$thres_disk\n" if ($disk);

	for (my $i = 0; $i <= $#proc; $i++) {
		print OUTCFG "proc_$proc[$i]:$thres_max_proc[$i]:$thres_min_proc[$i]\n";
	}
	for (my $i = 0; $i <= $#queue; $i++) {
		my $p = $queue[$i];
		print OUTCFG "queue:$queue[$i]:$thres_max_queue[$i]\n";
	}
	if ($hddtemp) {
		foreach (split(/[,;\s]+/, $hddtemp)) {
			print OUTCFG "hddtemp:$_:$thres_hddtemp\n";
		}
	}
	for (my $i = 0; $i <= $#dev; $i++) {
		my $p = $dev[$i];
		print OUTCFG "dev:$dev[$i]:$thres_max_dev[$i]\n";
	}
	close(OUTCFG);
}
unless(open(INST, ">install_all.sh")) {
	print "\nError: can't write post install file install_all.sh, $!\n";
	exit 0;
}
print INST qq{#!/bin/sh
if [ ! -d "$bin_dir" ]; then
	mkdir -p $bin_dir
fi
if [ ! -d "$base_dir" ]; then
	mkdir -p $base_dir
fi
if [ ! -d "$conf_dir" ]; then
	mkdir -p $conf_dir
fi
if [ ! -d "$html_dir" ]; then
	mkdir -p $html_dir
fi
if [ -e $conf_dir/sysusage.cfg ]; then
	echo "File sysusage.cfg already exists. Rename it into sysusage.cfg.old"
	mv $conf_dir/sysusage.cfg $conf_dir/sysusage.cfg.old
fi
install -m 644 sysusage.cfg $conf_dir/
install -m 755 bin/sysusage $bin_dir/
install -m 755 bin/sysusagegraph $bin_dir/
install -m 755 bin/sysusagewarn $bin_dir/

echo "
Now you can run sysusage into you crontab as follow:

	*/1 * * * * $bin_dir/sysusage
	*/5 * * * * $bin_dir/sysusagegraph

"
};
close(INST);
`chmod 755 install_all.sh`;

WriteMakefile(
    'NAME'         => 'SysUsage::Sar',
    'VERSION_FROM' => 'Sar.pm',
    'dist'         => {
			'COMPRESS'=>'gzip -9f', 'SUFFIX' => 'gz',
			'ZIP'=>'/usr/bin/zip','ZIPFLAGS'=>'-rl'
		      },
    'AUTHOR'       => 'Gilles Darold (gilles@darold.net)',
    'ABSTRACT'     => 'System Usage Monitoring',
    'clean'        => {FILES => "install_all.sh sysusage.cfg"}
);

sub MY::postamble {
    my $postamble = <<'END';
install_all: install_all.sh
	sh install_all.sh
END
    $postamble;
}

print "Done...\n\n";
print "Now type 'make && make install && make install_all'\n";
print "To learn howto use sysusage take a look at the README\n";
print "and INSTALL files\n";
print "Enjoy.\n";
